/**
#The Toce urban case
 
## Before you start
Before you start, be sure to create the following folders in the same place:
"Movies" "Pic" "Raster" "Gauges" 


To compile, type in the terminal: 
qcc Case-toce-urb.c -o Case-toce-urb.x -lm ~/basilisk/src/kdt/kdt.o
*/

/**
## Include and def
*/
#include "b-flood/saint-venant-topo.h"
#include "b-flood/manning.h"
#include "b-flood/inflow.h"
#include "terrain.h"
#include "input.h"

#define MAXLEVEL 10 // 1024 Volume
#define MINLEVEL 6
#define EMAXE 1e-4

int nf,nc,ncell;
timer t0;

/**
## Parameters  
*/
int main() {
  n = 0.0162;
  L0 = 14.9;
  X0 = 0.1;
  Y0 = 0;
  G = 9.81;
  dry = 1e-6;
  N = 1 << MAXLEVEL; // Which is equivalent to N = 2^(maxlevel)
  run();
}

event stop(t = 60)
  fprintf(stderr,"fini\n");


/**
## Initial condition
   
*/
event init (i = 0){
  terrain (zb, "./Topo/topo-a", NULL);
  DT = 0.05;
  t0 = timer_start();
  double dx = L0/N;
 foreach(){
    if( y <= 8.4 && y >= 5 && x <= 32*dx)
      river[] = 1;
  }
  boundary({zb,river});

  static FILE * rpz = fopen ("./Raster/raster_zb-stag.asc", "w");
  output_grd (zb, rpz);
}

/**
## Inflow boundary condition

On the left, we impose the flow rate using the "discharge" function, 
 */
u.t[left] = dirichlet(0);
u.n[left] = neumann(0);

double l1, eta1, vola = 0;
event bound_left( i++ ){
  l1 =  hydrograph ( "./hydro.asc", 1);
  if( l1 < 0 )
    l1 = 0;
  l1 /= 1000.; // convertir les litres en metres cube
  eta1 = eta_b (l1,left,1);
  
  h[left] = dirichlet( max( river[] == 1 ? eta1 - zb[] : 0 , 0) );
  eta[left] = dirichlet( max( river[] == 1 ? eta1 : zb[]  , zb[]) );

  double vol = 0;
  foreach(reduction(+:vol)){
    if( h[] > dry )
      vol += h[] * sq(Delta);
  }
  double Q = 0;
  if( dt > 0 )
    Q =  (vol- vola)/dt;
  vola = vol;
 
  static FILE * fpp = fopen("eta.dat","w");
  if( i == 0)
     fprintf(fpp,"#t Q Qread EtaImp\n");
  fprintf(fpp,"%lf %lf %lf %lf \n",t,Q,l1,eta1);
  fflush(fpp);
  
}

/**
## Outflow boundary condition
We set a free exit condition on the right edge of the domain.
*/

u.t[right] = neumann(0);
u.n[right] = neumann(0);
h[right] = neumann(0);
eta[right] = neumann(0);


/**
## Adaptative refinement
   
*/
int adapt_H() {
  scalar nh[];
  foreach()
    nh[] = h[] > dry ? h[] : 0;
  boundary ({nh});
  astats s = adapt_wavelet ({nh}, (double[]){EMAXE} ,MAXLEVEL,MINLEVEL);
  nc += s.nc;
  nf += s.nf;
  return s.nf;
}
event do_adapt_H (i++){
  if( t > 1 )
    adapt_H();
}
//////// OUTPUT
/**
## Logfile
*/

event logfile (i += 10) {
  timing tr = timer_timing (t0, i, 0, NULL);
  stats s = statsf (h);
  scalar v[];
  foreach()
    v[] = norm(u);
  stats no = statsf (v);
  if( i == 0 )
    fprintf (stderr, "#1t 2treal 3i 4h.min 5h.max 6h.sum 7u.min 8u.max 9dt\n");  
  fprintf (stderr, "%g %g %d %g %g %g %g %g %g\n", t,tr.real, i, s.min, s.max, s.sum, 
	   no.min, no.max, dt);
}

Gauge gauges[] = {  
  // gauge   x         y       zb
  //{"./Gauges/P1",-0.572923,7.618307, "P1"}, 
  {"./Gauges/P2",0.280560,7.585584, "P2"}, 
  {"./Gauges/P3",3.279013,7.324738, "P3"}, 
  {"./Gauges/P4",3.153249,7.834554, "P4"}, 
  {"./Gauges/P5",3.534738,7.748334, "P5"}, 
  {"./Gauges/P6",3.916227,7.662114, "P6"}, 
  {"./Gauges/P7",4.128212,7.533788, "P7"}, 
  {"./Gauges/P8",4.044554,7.874098, "P8"}, 
  {"./Gauges/P9",4.256538,7.745772, "P9"}, 
  {"./Gauges/P10",4.638028,7.659552, "P10"}, 
  {NULL}
};
event gauges1 (t += 0.2) output_gauges (gauges, {h});

/**
## Movies

We record movies of the water depth, the velocity and the level of
refinement.*/
event movies ( t += 0.1 ) {
  scalar m[], v[], fr[];
  
  foreach(){
    v[] = norm(u);
    m[] = (h[] > dry ) - 0.5;
    fr[] = h[] > dry ? v[]/sqrt(G*h[]) : 0;
  }    
  boundary ({m,v,fr});
  
  static FILE * fph = popen ("ppm2mpeg > ./Movies/height.mpg", "w");
  output_ppm (h, fph, mask = m, min = 0, max = 0.2, n = N, linear = true);
  
  static FILE * fpv = popen ("ppm2mpeg > ./Movies/vel.mpg", "w");
  output_ppm (v, fpv, mask = m, min = 0, max = 4, n = N, linear = true);
  
  static FILE * fpfr = popen ("ppm2mpeg > ./Movies/froude.mpg", "w");
  output_ppm (fr, fpfr, mask = m, min = 0, max = 2, n = N, linear = true);
  
  static FILE * fp1 = popen ("ppm2mpeg > ./Movies/level.mpg", "w");
  scalar l = m;
  foreach()
    l[] = level;
  output_ppm (l, fp1, min = MINLEVEL, max = MAXLEVEL, n = N);
}


/**
## Fractal dim 

*/
int ifractal = 0;
event fracdim( i += 10){
  int ncell = 0;
  foreach(reduction(+:ncell))
    ncell++;
  static FILE * ffrac = fopen("fdim.dat","w");
  if( ifractal == 0){
    fprintf(ffrac,"# t \t i \t Ncell \t Nrefined \t Ncoarsen");
    ifractal++;
    fprintf(ffrac,"%.1lf \t %i \t %i \t %i \t %i\n",t,i,ncell,nf,nc);
  nf = nc = 0;
  }
}

event snap(t = 30){
  static FILE * rph = fopen ("./Raster/raster_h-t30.asc", "w");
  output_grd (h, rph);

  scalar m[], v[];
    
  foreach(){
    v[] = norm(u);
    m[] = (h[] > dry ) - 0.5;
  }    
  boundary ({m,v});
  
  static FILE * rpu = fopen ("./Raster/raster_v-t30.asc", "w");
  output_grd (v, rpu);
}

event ending(t = end){
  static FILE * rph = fopen ("./Raster/raster_h.asc", "w");
  output_grd (h, rph);

  scalar m[], v[];
    
  foreach(){
    v[] = norm(u);
    m[] = (h[] > dry ) - 0.5;
  }    
  boundary ({m,v});
  
  static FILE * rpu = fopen ("./Raster/raster_v.asc", "w");
  output_grd (v, rpu);
}

 
event figures (t += 1)
{
  scalar m[], etam[];
  foreach() {
    etam[] = eta[]*(h[] > dry);
    m[] = etam[] - zb[];
  }
  boundary ({m, etam});
  char name[80];
  sprintf (name, "./Pic/h-%g-1.png", t);
  output_ppm (h, mask = m, min = 0, max = 0.2, file = name, n = 1024,
	      linear = true,
	      opt = "-fill white -opaque black");
  sprintf (name, "./Pic/h-%g-2.png", t);
  output_ppm (h, mask = m, min = 0, max = 0.2, file = name, n = 1024,
	      linear = true);
  
  
  sprintf (name, "./Pic/level-%g.png", t);
  scalar l[];
  foreach()
    l[] = level;
  output_ppm (l, min = MINLEVEL, max = MAXLEVEL, file = name, n = 1024,
	      linear = false,);
}
/**
## Link to the homepage
* [Homepage](http://basilisk.fr/sandbox/B-flood/Readme)	    		  
   		  
*/